/* 
 * tcltds.c
 *
 * This file contains a simple Tcl package "tds" that is intended
 * for providing tds control services.
 *
 * Copyright (c) 2003 Kazuro Furukawa
 *
 * Apr.11.2003. k.furukawa, first version
 *
 * Sep.5.2003.  k.f, add lock and unlock procedures
 * Sep.6.2003.  k.f, add timeout procedure
 */
static char what_tcltds[] =
"@(#)tcltds.c, v1.1.2, k.furukawa, tcl tds control, Apr.2003-Sep.2003";

#include "tcl.h"
#include <stdio.h>
#include <rpc/rpc.h>
#include "vxi11.h"
#include "tdslib.h"

/*
 * Prototypes for procedures defined later in this file:
 */

static int	Tds_OpenCmd _ANSI_ARGS_((ClientData clientData,
		    Tcl_Interp *interp, int argc, char **argv));
static int	Tds_TimeoutCmd _ANSI_ARGS_((ClientData clientData,
		    Tcl_Interp *interp, int argc, char **argv));
static int	Tds_WriteCmd _ANSI_ARGS_((ClientData clientData,
		    Tcl_Interp *interp, int argc, char **argv));
static int	Tds_ReadCmd _ANSI_ARGS_((ClientData clientData,
		    Tcl_Interp *interp, int argc, char **argv));
static int	Tds_LockCmd _ANSI_ARGS_((ClientData clientData,
		    Tcl_Interp *interp, int argc, char **argv));
static int	Tds_UnlockCmd _ANSI_ARGS_((ClientData clientData,
		    Tcl_Interp *interp, int argc, char **argv));
static int	Tds_CloseCmd _ANSI_ARGS_((ClientData clientData,
		    Tcl_Interp *interp, int argc, char **argv));

#define BUFSIZE 1024
static char tdsprint[BUFSIZE*10];
static char tdserror[BUFSIZE];

/*
 * Tds_OpenCmd
 *
 *	This procedure is invoked to process the "Tds_open" Tcl command.
 *	It expects one argument and returns a status.  On success, 
 *	it returns a TDS identifier that may be used in future 
 *	invocations of commands like Tds_write, etc.
 *
 * Results:
 *	A standard Tcl result.
 */
static int
Tds_OpenCmd(dummy, interp, argc, argv)
    ClientData dummy;			/* Not used. */
    Tcl_Interp *interp;			/* Current interpreter. */
    int argc;				/* Number of arguments. */
    char **argv;			/* Argument strings. */
{
    tds_struct *ts;

#ifdef DEBUG
    {
	int i;
	for (i=0;i<argc;i++) printf("%s ",argv[i]);
	printf("\n"); fflush(stdout);
    }
#endif
    if (argc != 2) {
	Tcl_AppendResult(interp, "wrong # args: should be \"tds_open tdshost\"",
			 (char *) NULL);
	return TCL_ERROR;
    }

    if (tds_init( &ts, argv[1] )) {
	Tcl_AppendResult(interp, "couldn't initialize connection to \"", 
			 argv[1], "\"", (char *) NULL);
	return TCL_ERROR;
    }
    if (tds_open( ts )) {
	Tcl_AppendResult(interp, "couldn't open connection to \"", 
			 argv[1], "\"", (char *) NULL);
	return TCL_ERROR;
    }
    sprintf(tdsprint, "tds%p", ts);
    Tcl_SetResult(interp, tdsprint, TCL_STATIC);
    return TCL_OK;
}

/*
 * Tds_TimeoutCmd
 *
 *	This procedure is invoked to process the "Tds_timeout" Tcl command.
 *	It expects one argument, a TDS identifier, and returns a status.
 *
 * Results:
 *	A standard Tcl result.
 */
static int
Tds_TimeoutCmd(dummy, interp, argc, argv)
    ClientData dummy;			/* Not used. */
    Tcl_Interp *interp;			/* Current interpreter. */
    int argc;				/* Number of arguments. */
    char **argv;			/* Argument strings. */
{
    tds_struct *ts;
    int milliwait;

#ifdef DEBUG
    {
	int i;
	for (i=0;i<argc;i++) printf("%s ",argv[i]);
	printf("\n"); fflush(stdout);
    }
#endif
    if (argc != 3) {
	Tcl_AppendResult(interp, 
			 "wrong # args: should be \"tds_timeout tds-id milli-wait\"",
			 (char *) NULL);
	return TCL_ERROR;
    }

    if (sscanf(argv[1], "tds%p", &ts) != 1) {
	Tcl_AppendResult(interp, "can not find id. named \"", argv[1], 
			 "\"", (char *) NULL);
	return TCL_ERROR;
    }
    
    if (sscanf(argv[2], "%d", &milliwait) != 1) {
	Tcl_AppendResult(interp, "can not convert milli-wait \"", argv[2], 
			 "\"", (char *) NULL);
	return TCL_ERROR;
    }
    
    if (tds_timeout( ts, milliwait )) {
	Tcl_AppendResult(interp, "couldn't specify timeout \"", argv[2], 
			 "\" for \"", argv[1], "\"", (char *) NULL);
	return TCL_ERROR;
    }
    Tcl_SetResult(interp, "", TCL_STATIC);
    return TCL_OK;
}

/*
 * Tds_WriteCmd
 *
 *	This procedure is invoked to process the "Tds_write" Tcl command.
 *	It expects two argument, a TDS identifier and a string to be written, 
 *	and returns a status.
 *
 * Results:
 *	A standard Tcl result.
 */
static int
Tds_WriteCmd(dummy, interp, argc, argv)
    ClientData dummy;			/* Not used. */
    Tcl_Interp *interp;			/* Current interpreter. */
    int argc;				/* Number of arguments. */
    char **argv;			/* Argument strings. */
{
    tds_struct *ts;

#ifdef DEBUG
    {
	int i;
	for (i=0;i<argc;i++) printf("%s ",argv[i]);
	printf("\n"); fflush(stdout);
    }
#endif
    if (argc != 3) {
	Tcl_AppendResult(interp, 
			 "wrong # args: should be \"tds_write tds-id string\"",
			 (char *) NULL);
	return TCL_ERROR;
    }

    if (sscanf(argv[1], "tds%p", &ts) != 1) {
	Tcl_AppendResult(interp, "can not find id. named \"", argv[1], 
			 "\"", (char *) NULL);
	return TCL_ERROR;
    }
    
    if (tds_write( ts, argv[2] )) {
	Tcl_AppendResult(interp, "couldn't write \"", argv[1], "\"",
			 (char *) NULL);
	return TCL_ERROR;
    }
    Tcl_SetResult(interp, "", TCL_STATIC);
    return TCL_OK;
}

/*
 * Tds_ReadCmd
 *
 *	This procedure is invoked to process the "Tds_read" Tcl command.
 *	It expects one argument, a TDS identifier, and returns a status.
 *	On success, it returns a string which is read from TDS. 
 *
 * Results:
 *	A standard Tcl result.
 */
static int
Tds_ReadCmd(dummy, interp, argc, argv)
    ClientData dummy;			/* Not used. */
    Tcl_Interp *interp;			/* Current interpreter. */
    int argc;				/* Number of arguments. */
    char **argv;			/* Argument strings. */
{
    tds_struct *ts;

#ifdef DEBUG
    {
	int i;
	for (i=0;i<argc;i++) printf("%s ",argv[i]);
	printf("\n"); fflush(stdout);
    }
#endif
    if (argc != 2) {
	Tcl_AppendResult(interp, 
			 "wrong # args: should be \"tds_read tds-id\"",
			 (char *) NULL);
	return TCL_ERROR;
    }

    if (sscanf(argv[1], "tds%p", &ts) != 1) {
	Tcl_AppendResult(interp, "can not find id. named \"", argv[1], 
			 "\"", (char *) NULL);
	return TCL_ERROR;
    }
    
    if (tds_read( ts, tdsprint )) {
	Tcl_AppendResult(interp, "couldn't read \"", argv[1], "\"",
			 (char *) NULL);
	return TCL_ERROR;
    }
    Tcl_SetResult(interp, tdsprint, TCL_STATIC);
    return TCL_OK;
}

/*
 * Tds_LockCmd
 *
 *	This procedure is invoked to process the "Tds_lock" Tcl command.
 *	It expects one argument, a TDS identifier, and returns a status.
 *
 * Results:
 *	A standard Tcl result.
 */
static int
Tds_LockCmd(dummy, interp, argc, argv)
    ClientData dummy;			/* Not used. */
    Tcl_Interp *interp;			/* Current interpreter. */
    int argc;				/* Number of arguments. */
    char **argv;			/* Argument strings. */
{
    tds_struct *ts;

#ifdef DEBUG
    {
	int i;
	for (i=0;i<argc;i++) printf("%s ",argv[i]);
	printf("\n"); fflush(stdout);
    }
#endif
    if (argc != 2) {
	Tcl_AppendResult(interp, 
			 "wrong # args: should be \"tds_lock tds-id\"",
			 (char *) NULL);
	return TCL_ERROR;
    }

    if (sscanf(argv[1], "tds%p", &ts) != 1) {
	Tcl_AppendResult(interp, "can not find id. named \"", argv[1], 
			 "\"", (char *) NULL);
	return TCL_ERROR;
    }
    
    if (tds_lock( ts )) {
	Tcl_AppendResult(interp, "couldn't lock \"", argv[1], "\"",
			 (char *) NULL);
	return TCL_ERROR;
    }
    Tcl_SetResult(interp, "", TCL_STATIC);
    return TCL_OK;
}

/*
 * Tds_UnlockCmd
 *
 *	This procedure is invoked to process the "Tds_unlock" Tcl command.
 *	It expects one argument, a TDS identifier, and returns a status.
 *
 * Results:
 *	A standard Tcl result.
 */
static int
Tds_UnlockCmd(dummy, interp, argc, argv)
    ClientData dummy;			/* Not used. */
    Tcl_Interp *interp;			/* Current interpreter. */
    int argc;				/* Number of arguments. */
    char **argv;			/* Argument strings. */
{
    tds_struct *ts;

#ifdef DEBUG
    {
	int i;
	for (i=0;i<argc;i++) printf("%s ",argv[i]);
	printf("\n"); fflush(stdout);
    }
#endif
    if (argc != 2) {
	Tcl_AppendResult(interp, 
			 "wrong # args: should be \"tds_unlock tds-id\"",
			 (char *) NULL);
	return TCL_ERROR;
    }

    if (sscanf(argv[1], "tds%p", &ts) != 1) {
	Tcl_AppendResult(interp, "can not find id. named \"", argv[1], 
			 "\"", (char *) NULL);
	return TCL_ERROR;
    }
    
    if (tds_unlock( ts )) {
	Tcl_AppendResult(interp, "couldn't unlock\"", argv[1], "\"",
			 (char *) NULL);
	return TCL_ERROR;
    }
    Tcl_SetResult(interp, "", TCL_STATIC);
    return TCL_OK;
}

/*
 * Tds_CloseCmd
 *
 *	This procedure is invoked to process the "Tds_close" Tcl command.
 *	It expects one argument, a TDS identifier, and returns a status.
 *
 * Results:
 *	A standard Tcl result.
 */
static int
Tds_CloseCmd(dummy, interp, argc, argv)
    ClientData dummy;			/* Not used. */
    Tcl_Interp *interp;			/* Current interpreter. */
    int argc;				/* Number of arguments. */
    char **argv;			/* Argument strings. */
{
    tds_struct *ts;

#ifdef DEBUG
    {
	int i;
	for (i=0;i<argc;i++) printf("%s ",argv[i]);
	printf("\n"); fflush(stdout);
    }
#endif
    if (argc != 2) {
	Tcl_AppendResult(interp, 
			 "wrong # args: should be \"tds_close tds-id\"",
			 (char *) NULL);
	return TCL_ERROR;
    }

    if (sscanf(argv[1], "tds%p", &ts) != 1) {
	Tcl_AppendResult(interp, "can not find id. named \"", argv[1], 
			 "\"", (char *) NULL);
	return TCL_ERROR;
    }
    
    if (tds_close( ts )) {
	Tcl_AppendResult(interp, "couldn't close\"", argv[1], "\"",
			 (char *) NULL);
	return TCL_ERROR;
    }
    Tcl_SetResult(interp, "", TCL_STATIC);
    return TCL_OK;
}

/*
 * Tds_Init --
 *
 *	This is a package initialization procedure, which is called
 *	by Tcl when this package is to be added to an interpreter.
 *
 * Results:
 *	None.
 */

int
Tds_Init(interp)
    Tcl_Interp *interp;		/* Interpreter in which the package is
				 * to be made available. */
{
    int code;

    code = Tcl_PkgProvide(interp, "Tds", "1.1");
    if (code != TCL_OK) {
	return code;
    }
#ifdef DEBUG
    printf("tds_init\n");
    fflush(stdout);
#endif
    Tcl_CreateCommand(interp, "Tds_open", Tds_OpenCmd, (ClientData) 0,
	    (Tcl_CmdDeleteProc *) NULL);
    Tcl_CreateCommand(interp, "Tds_timeout", Tds_TimeoutCmd, (ClientData) 0,
	    (Tcl_CmdDeleteProc *) NULL);
    Tcl_CreateCommand(interp, "Tds_write", Tds_WriteCmd, (ClientData) 0,
	    (Tcl_CmdDeleteProc *) NULL);
    Tcl_CreateCommand(interp, "Tds_read", Tds_ReadCmd, (ClientData) 0,
	    (Tcl_CmdDeleteProc *) NULL);
    Tcl_CreateCommand(interp, "Tds_lock", Tds_LockCmd, (ClientData) 0,
	    (Tcl_CmdDeleteProc *) NULL);
    Tcl_CreateCommand(interp, "Tds_unlock", Tds_UnlockCmd, (ClientData) 0,
	    (Tcl_CmdDeleteProc *) NULL);
    Tcl_CreateCommand(interp, "Tds_close", Tds_CloseCmd, (ClientData) 0,
	    (Tcl_CmdDeleteProc *) NULL);
    return TCL_OK;
}

/*
 * Tds_SafeInit --
 *
 *	This is a package initialization procedure, which is called
 *	by Tcl when this package is to be added to an unsafe interpreter.
 *
 * Results:
 *	None.
 */

int
Tds_SafeInit(interp)
    Tcl_Interp *interp;		/* Interpreter in which the package is
				 * to be made available. */
{
    return TCL_OK;
}

